=== modified file 'NEWS'
--- NEWS	2011-09-08 00:33:28 +0000
+++ NEWS	2011-11-08 16:29:37 +0000
@@ -42,6 +42,13 @@
     - Fix support for displaying foreign revision ids.
       (Jelmer Vernooij, #736026)
 
+    - Add hook 'controller' to BranchWSGIApp, allowing other bzr plugins
+      to provide custom branch-specific controllers. (Jelmer Vernooij, #585822)
+
+    - Add privacy notification code to loggerhead, allowing branches to be
+      marked as private and a notification ribbon to be displayed via
+      javascript on the view pages. (JC Sackett, #823471)
+
 
 1.18.1 [24Mar2011]
 ------------------

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2011-08-17 11:04:00 +0000
+++ loggerhead/apps/branch.py	2011-11-04 18:16:45 +0000
@@ -22,6 +22,7 @@
 
 import bzrlib.branch
 import bzrlib.errors
+from bzrlib.hooks import Hooks
 import bzrlib.lru_cache
 
 from paste import request
@@ -49,7 +50,7 @@
 
     def __init__(self, branch, friendly_name=None, config={},
                  graph_cache=None, branch_link=None, is_root=False,
-                 served_url=_DEFAULT, use_cdn=False):
+                 served_url=_DEFAULT, use_cdn=False, private=False):
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
@@ -61,6 +62,13 @@
         self.is_root = is_root
         self.served_url = served_url
         self.use_cdn = use_cdn
+        self.private = private
+
+    def public_private_css(self):
+        if self.private:
+            return "private"
+        else:
+            return "public"
 
     def get_history(self):
         revinfo_disk_cache = None
@@ -155,6 +163,10 @@
                     self.served_url = self.url([])
                 except bzrlib.errors.InvalidURL:
                     self.served_url = None
+        for hook in self.hooks['controller']:
+            controller = hook(self, environ)
+            if controller is not None:
+                return controller
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
@@ -165,9 +177,9 @@
             environ['loggerhead.as_json'] = True
             path = request.path_info_pop(environ)
         cls = self.controllers_dict.get(path)
-        if cls is None:
-            raise httpexceptions.HTTPNotFound()
-        return cls(self, self.get_history)
+        if cls is not None:
+            return cls(self, self.get_history)
+        raise httpexceptions.HTTPNotFound()
 
     def app(self, environ, start_response):
         self.branch.lock_read()
@@ -181,3 +193,23 @@
                 raise
         finally:
             self.branch.unlock()
+
+
+class BranchWSGIAppHooks(Hooks):
+    """A dictionary mapping hook name to a list of callables for WSGI app branch hooks.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+        """
+        Hooks.__init__(self, "bzrlib.plugins.loggerhead.apps.branch",
+            "BranchWSGIApp.hooks")
+        self.add_hook('controller',
+            "Invoked when looking for the controller to use for a "
+            "branch subpage. The api signature is (branch_app, environ)."
+            "If a hook can provide a controller, it should return one, "
+            "as a standard WSGI app. If it can't provide a controller, "
+            "it should return None", (1, 19))
+
+
+BranchWSGIApp.hooks = BranchWSGIAppHooks()

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2011-04-22 17:22:49 +0000
+++ loggerhead/static/css/global.css	2011-10-25 15:58:27 +0000
@@ -343,3 +343,64 @@
 #footer {
     font-size: 77%;
     }
+
+    
+/* ====================
+   Global notifications
+*/
+/* Move the content down so the notification banner doesn't hide any content. */
+body.global-notification-visible {
+    padding-top: 40px;
+    }
+body.global-notification-visible .login-logout {
+    top: 45px;
+    }
+.global-notification {
+    position: fixed;
+    z-index: 10;
+    top: 0;
+    left: 0;
+    right: 0;
+    padding: 8px 20px;
+    /* Define colour for browsers that don't support transparency */
+    background-color: #8d1f1f;
+    /* Set transparent background for browsers that support it */
+    background-color: rgba(125,0,0,0.9);
+    color: #fff;
+    text-shadow: 0 -1px 0 #631616;
+    font-size: 14px;
+    line-height: 21px;
+    font-weight: bold;
+    -moz-box-shadow: 0 0 5px #333;
+    -webkit-box-shadow: 0 0 5px #333;
+    box-shadow: 0 0 5px #333;
+    }
+.global-notification .sprite.notification-private {
+    float: left;
+    display: inline-block;
+    height: 21px;
+    width: 20px;
+    margin-right: 10px;
+    padding: 0;
+    }
+.global-notification-close, .global-notification-close:active,
+.global-notification-close:visited {
+    color: #e47a7a;
+    }
+.global-notification-close {
+    display: block;
+    position: absolute;
+    top: 11px;
+    right: 20px;
+    font-size: 12px;
+    font-weight: normal;
+    line-height: 14px;
+    }
+.global-notification-close .sprite.notification-close {
+    float: right;
+    display: block;
+    height: 9px;
+    width: 8px;
+    margin: 3px 0 0 7px;
+    padding: 0;
+    }

=== modified file 'loggerhead/static/javascript/custom.js'
--- loggerhead/static/javascript/custom.js	2009-04-01 15:28:21 +0000
+++ loggerhead/static/javascript/custom.js	2011-10-26 16:06:54 +0000
@@ -1,4 +1,4 @@
-Y = YUI().use("node", "io-base", "anim");
+Y = YUI().use("base", "node", "io-base", "anim");
 
 var global_timeout_id = null;
 var global_search_request = null;
@@ -280,3 +280,89 @@
   }
 };
 
+var notification_node = null;
+/*
+ * Display privacy notifications
+ *
+ * This should be called after the page has loaded e.g. on 'domready'.
+ */
+function setup_privacy_notification(config) {
+    if (notification_node !== null) {
+        return;
+    }
+    var notification_text = 'The information on this page is private';
+    var hidden = true;
+    var target_id = "loggerheadCont";
+    if (config !== undefined) {
+        if (config.notification_text !== undefined) {
+            notification_text = config.notification_text;
+        }
+        if (config.hidden !== undefined) {
+            hidden = config.hidden;
+        }
+        if (config.target_id !== undefined) {
+            target_id = config.target_id;
+        }
+    }
+    var id_selector = "#" + target_id;
+    var main = Y.get(id_selector);
+    notification_node = Y.Node.create('<div></div>')
+        .addClass('global-notification');
+    if (hidden) {
+        notification_node.addClass('hidden');
+    }
+    var notification_span = Y.Node.create('<span></span>')
+        .addClass('sprite')
+        .addClass('notification-private');
+    notification_node.set('innerHTML', notification_text);
+    main.appendChild(notification_node);
+    notification_node.appendChild(notification_span);
+};
+
+function display_privacy_notification() {
+    /* Set a temporary class on the body for the feature flag,
+     this is because we have no way to use feature flags in
+     css directly. This should be removed if the feature
+     is accepted. */
+    var body = Y.get('body');
+    body.addClass('feature-flag-bugs-private-notification-enabled');
+    // Set the visible flag so that the content moves down.
+    body.addClass('global-notification-visible');
+
+    setup_privacy_notification();
+    var global_notification = Y.get('.global-notification');
+    if (global_notification.hasClass('hidden')) {
+        global_notification.addClass('transparent');
+        global_notification.removeClass('hidden');
+
+        var fade_in = new Y.Anim({
+            node: global_notification,
+            to: {opacity: 1},
+            duration: 0.3
+        });
+        var body_space = new Y.Anim({
+            node: 'body',
+            to: {'paddingTop': '40px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+        var black_link_space = new Y.Anim({
+            node: '.black-link',
+            to: {'top': '45px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+
+        fade_in.run();
+        body_space.run();
+        black_link_space.run();
+    }
+};
+
+Y.on('domready', function() {
+    var body = Y.get('body');
+    if (body.hasClass('private')) {
+        setup_privacy_notification();
+        display_privacy_notification();
+    }
+});

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2011-03-03 10:58:57 +0000
+++ loggerhead/templates/macros.pt	2011-11-04 18:16:32 +0000
@@ -43,7 +43,7 @@
     <metal:block metal:define-slot="header_extras" />
   </head>
 
-<body>
+ <body tal:attributes="class python:branch.public_private_css()">
     <!-- Loggerhead Content Area -->
       
       <div metal:define-slot="backlink" />

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2011-09-02 09:43:08 +0000
+++ loggerhead/tests/test_controllers.py	2011-11-02 01:56:02 +0000
@@ -1,11 +1,7 @@
-import simplejson
-
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
-from loggerhead.controllers.revision_ui import RevisionUI
 from loggerhead.tests.test_simple import BasicTests, consume_app
-from loggerhead import util
 
 
 class TestInventoryUI(BasicTests):
@@ -168,6 +164,7 @@
         annotated = annotate_info['annotated']
         self.assertEqual(0, len(annotated))
 
+
 class TestFileDiffUI(BasicTests):
 
     def make_branch_app_for_filediff_ui(self):
@@ -233,3 +230,31 @@
         revlog_ui = branch_app.lookup_app(env)
         self.assertOkJsonResponse(revlog_ui, env)
 
+
+class TestControllerHooks(BasicTests):
+
+    def test_dummy_hook(self):
+        return
+        # A hook that returns None doesn't influence the searching for
+        # a controller.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: None
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertRaises(KeyError, app.lookup_app, env)
+
+    def test_working_hook(self):
+        # A hook can provide an app to use for a particular request.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: "I am hooked"
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertEquals("I am hooked", app.lookup_app(env))

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2011-09-08 00:33:28 +0000
+++ loggerhead/tests/test_simple.py	2011-10-31 21:08:11 +0000
@@ -56,8 +56,8 @@
         self.assertEqual(None, start[2])
         simplejson.loads(content)
 
-    def make_branch_app(self, branch):
-        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name')
+    def make_branch_app(self, branch, **kw):
+        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name', **kw)
         branch_app._environ = {
             'wsgi.url_scheme':'',
             'SERVER_NAME':'',
@@ -84,6 +84,12 @@
         self.msg = 'a very exciting commit message <'
         self.revid = self.tree.commit(message=self.msg)
 
+    def test_public_private(self):
+        app = self.make_branch_app(self.tree.branch, private=True)
+        self.assertEqual(app.public_private_css(), 'private')
+        app = self.make_branch_app(self.tree.branch)
+        self.assertEqual(app.public_private_css(), 'public')
+
     def test_changes(self):
         app = self.setUpLoggerhead()
         res = app.get('/changes')

